////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.program.android.chumsgroup;

import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.util.EntityUtils;
import org.watto.android.Settings;
import org.watto.android.WSAboutScreen;
import org.watto.android.WSSettingsChanger;
import org.watto.android.component.WSAndroidPopup;
import org.watto.android.event.WSViewClickableInterface;
import org.watto.android.event.listener.WSViewClickableListener;
import org.watto.program.android.chumsgroup.http.HttpMultipartMode;
import org.watto.program.android.chumsgroup.http.MultipartEntity;
import org.watto.program.android.chumsgroup.http.content.StringBody;
import android.app.Activity;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.Intent;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;


/***********************************************************************************************
Calculates the Weight Points value for a food product
***********************************************************************************************/
public class LoginWindow extends Activity implements WSViewClickableInterface {

  /***********************************************************************************************
  Performs an action when a single-click event is triggered on a <code>View</code>
  @param source the <code>View</code> that triggered the event
  @return <b>true</b> if the event was handled by this class<br />
          <b>false</b> if the event wasn't handled by this class, and thus should be passed on to
          the parent class for handling.
  ***********************************************************************************************/
  public boolean onClick(View source){
    int id = source.getId();
    if (id == R.id.button_login) {
      runLoginThread();
    }
    else {
      return false;
    }
    return true;
  }


  /***********************************************************************************************
  Creates the <code>Activity</code> from the XML file
  @param savedInstanceState the saved state of the <code>Activity</code>
  ***********************************************************************************************/
  public void onCreate(Bundle savedInstanceState){
    super.onCreate(savedInstanceState);

    // Load and prepare the settings, if the application has only just been loaded
    if (!Settings.hasApplication()) {
      Settings.setApplication(this);
    }

    // Creates the view from the XML file
    setContentView(R.layout.login_window);
    

    // Load the remembered values for the text boxes
    String username = Settings.get("username");
    String password = Settings.get("password");
    
    ((EditText)findViewById(R.id.tf_username)).setText(username);
    ((EditText)findViewById(R.id.tf_password)).setText(password);

    
    // Set the View Click Listeners
    WSViewClickableListener viewClickableListener = new WSViewClickableListener(this);
    ((Button)findViewById(R.id.button_login)).setOnClickListener(viewClickableListener);
    
    
    // Log in automatically (if set in the Settings)
    boolean autoLogin = Settings.getBoolean("AutoLogin");
    if (autoLogin && (username != null && !username.equals("")) && (password != null && !password.equals(""))){
      runLoginThread();
    }


  }
  
  



  /***********************************************************************************************
  Creates the <code>Menu</code> from the XML file when the Options button is pressed
  @param menu the <code>Menu</code> <code>View</code> to populate with menu items
  @return <b>true</b> to display the menu, <br />
          <b>false</b> to keep the menu hidden.
  ***********************************************************************************************/
  public boolean onCreateOptionsMenu(Menu menu){
    MenuInflater inflater = getMenuInflater();
    inflater.inflate(R.menu.login_window,menu);
    return true;
  }


  /***********************************************************************************************
  Performs an action when a <code>MenuItem</code> is selected in the Options menu
  @param item the <code>MenuItem</code> that was selected
  @return <b>true</b> if the event was handled by this class<br />
          <b>false</b> if the event wasn't handled by this class, and thus should be passed on to
          the parent class for handling.
  ***********************************************************************************************/
  public boolean onOptionsItemSelected(MenuItem item){
    int id = item.getItemId();
    if (id == R.id.menu_login_window_about) {
      showAbout();
    }
    else if (id == R.id.menu_login_window_settings) {
      showSettings();
    }
    else if (id == R.id.menu_login_window_login) {
      runLoginThread();
    }
    else {
    return false;
    }
    return true;
  }


  /***********************************************************************************************
  Hides the screen fields whenever the <code>Activity</code> is brought to the foreground
  ***********************************************************************************************/
  public void onResume(){
    super.onResume();
    
    /*
    // Hide the screen fields according to the chosen formula
    hideScreenFields();

    // Sets the input fields as decimal or not, according to the settings
    setDecimalFields();
    */

  }
  
  final int DIALOG_LOADING = 1;
  
  /***********************************************************************************************
  Overwritten to handle change of screen orientation while the dialog is showing
  ***********************************************************************************************/
  protected Dialog onCreateDialog(int id) {
    if(id == DIALOG_LOADING){
    ProgressDialog loadingDialog = new ProgressDialog(this);
    loadingDialog.setMessage("Logging In...");
    loadingDialog.setIndeterminate(true);
    loadingDialog.setCancelable(true);
    return loadingDialog;
    }

    return super.onCreateDialog(id);
    }
  
  
  /***********************************************************************************************
  Overwritten to close the dialog when clicking the 'back' button and returning to this activity
  ***********************************************************************************************/
  protected void onSaveInstanceState(Bundle outState) {
    try {
    dismissDialog(DIALOG_LOADING);
    } catch (Exception e) {
    //ignore error
    }
    super.onSaveInstanceState(outState);
    }
  
  
 
  public Activity getThisActivity(){
    return this;
}
  
  /***********************************************************************************************

  ***********************************************************************************************/
  public void runLoginThread(){
   
    
    // "Loading" popup
    showDialog(DIALOG_LOADING);

    // call login() in a Thread
    new Thread(new Runnable(){
      public void run(){
      login();
      try {
      dismissDialog(DIALOG_LOADING);
      }
      catch (Throwable t){
      }
      }
      }).start();
  }
  
  
  /***********************************************************************************************

  ***********************************************************************************************/
  public void login(){

    
    try {
      
      String username = ((EditText)findViewById(R.id.tf_username)).getText().toString();
      String password = ((EditText)findViewById(R.id.tf_password)).getText().toString();
      
      if (username == null || username.equals("")){
        this.runOnUiThread(new Runnable(){
          public void run(){
            WSAndroidPopup.showError(getThisActivity(),R.string.error_username_missing);
          }
          });
        return;
      }
      if (password == null || password.equals("")){
        this.runOnUiThread(new Runnable(){
          public void run(){
            WSAndroidPopup.showError(getThisActivity(),R.string.error_password_missing);
          }
          });
        return;
      }
    
      // Remember the values for the text boxes (for populating the fields next time we run the app)
      Settings.set("username",username);
      Settings.set("password",password);

      
    
    // 1. Call a GET to http://chumsgroup.com/users/sign_in
    String queryUrl = "http://chumsgroup.com/users/sign_in";
    HttpGet request = new HttpGet(queryUrl);
    request.setHeader("Accept","text/html"); // force html response
    request.setHeader("Accept-Encoding","identity"); // force no-compression response (for 3G compat)

    DefaultHttpClient client = SharedHttpClient.getClient();
    HttpResponse response = client.execute(request);

    // 2. Get the authenticity token
    String token = "";
    if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
      String responseMessage = EntityUtils.toString(response.getEntity());
      String[] lines = responseMessage.split("\n");
      
      String searchToken = "name=\"csrf-token\"";
      
      for (int i=0;i<lines.length;i++){
        if (lines[i].contains(searchToken)){
          // found the token line
          int tokenPos = lines[i].indexOf("content=\"");
          tokenPos += 9;
          int endPos = lines[i].indexOf("\"",tokenPos);
          token = lines[i].substring(tokenPos,endPos);
          break;
        }
      }
    }
    
    if (token == null || token.equals("")){
      this.runOnUiThread(new Runnable(){
        public void run(){
          WSAndroidPopup.showError(getThisActivity(),R.string.error_token_invalid);
        }
        });
      return;
    }
    
    
    SharedHttpClient.setToken(token);
    
    
    // 3. Send a POST to http://chumsgroup.com/users/sign_in, passing in authenticity_token, user[email], user[password], user[remember_me]=0, commit="Chum in"
    String batchURL = "http://chumsgroup.com/users/sign_in";
    HttpPost postRequest = new HttpPost(batchURL);
    postRequest.setHeader("Accept","text/html"); // force html response
    postRequest.setHeader("Accept-Encoding","identity"); // force no-compression response (for 3G compat)

    // Create a new MultipartEntity for holding the POST data
    MultipartEntity entity = new MultipartEntity(HttpMultipartMode.BROWSER_COMPATIBLE);
    entity.addPart("authenticity_token",new StringBody(token));
    entity.addPart("user[email]",new StringBody(username));
    entity.addPart("user[password]",new StringBody(password));
    entity.addPart("user[remember_me]",new StringBody("0"));
    entity.addPart("commit",new StringBody("Chum in"));

    postRequest.setEntity(entity);

    // Send the request
    client = SharedHttpClient.getClient();
    response = client.execute(postRequest);

    // Get the response code
    if (response.getStatusLine().getStatusCode() != HttpStatus.SC_OK) {
      // Failure
      this.runOnUiThread(new Runnable(){
        public void run(){
          WSAndroidPopup.showError(getThisActivity(),R.string.error_credentials_invalid);
        }
        });
      return;
    }

    // 4. Hopefully get a webpage back, so load it.
    //String responseMessage = EntityUtils.toString(response.getEntity());
    //String temp = "";
    
    
    Intent postsActivity = new Intent(getBaseContext(),PostsWindow.class);
    startActivity(postsActivity);
    
    // so when we hit the back button on PostsWindow, it closes the whole application
    finish();
    
    }
    catch (Throwable t){
    }
  }
  
  
  /***********************************************************************************************
  Shows the about <code>Activity</code>
  ***********************************************************************************************/
  public void showAbout(){
    Intent aboutActivity = new Intent(getBaseContext(),WSAboutScreen.class);
    startActivity(aboutActivity);
  }


  /***********************************************************************************************
  Shows the settings <code>Activity</code>
  ***********************************************************************************************/
  public void showSettings(){
    Intent settingsActivity = new Intent(getBaseContext(),WSSettingsChanger.class);
    startActivity(settingsActivity);
  }

}