////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                            //
//                                       WATTO STUDIOS                                        //
//                             Java Code, Programs, and Software                              //
//                                    http://www.watto.org                                    //
//                                                                                            //
//                           Copyright (C) 2004-2010  WATTO Studios                           //
//                                                                                            //
// This program is free software; you can redistribute it and/or modify it under the terms of //
// the GNU General Public License published by the Free Software Foundation; either version 2 //
// of the License, or (at your option) any later versions. This program is distributed in the //
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranties //
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License //
// at http://www.gnu.org for more details. For updates and information about this program, go //
// to the WATTO Studios website at http://www.watto.org or email watto@watto.org . Thanks! :) //
//                                                                                            //
////////////////////////////////////////////////////////////////////////////////////////////////

package org.watto.program.android.chumsgroup;

import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.watto.android.WSAboutScreen;
import org.watto.android.WSSettingsChanger;
import org.watto.android.component.WSAndroidPopup;
import org.watto.android.event.WSViewClickableInterface;
import org.watto.android.event.listener.WSViewClickableListener;
import org.watto.program.android.chumsgroup.http.HttpMultipartMode;
import org.watto.program.android.chumsgroup.http.MultipartEntity;
import org.watto.program.android.chumsgroup.http.content.StringBody;
import android.app.Activity;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.Intent;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;


/***********************************************************************************************

***********************************************************************************************/
public class WritePostWindow extends Activity implements WSViewClickableInterface {

  /***********************************************************************************************
  Performs an action when a single-click event is triggered on a <code>View</code>
  @param source the <code>View</code> that triggered the event
  @return <b>true</b> if the event was handled by this class<br />
          <b>false</b> if the event wasn't handled by this class, and thus should be passed on to
          the parent class for handling.
  ***********************************************************************************************/
  public boolean onClick(View source){
    int id = source.getId();
    if (id == R.id.button_send_message) {
      runSendPostThread();
    }
    else {
      return false;
    }
    return true;
  }


  /***********************************************************************************************
  Creates the <code>Activity</code> from the XML file
  @param savedInstanceState the saved state of the <code>Activity</code>
  ***********************************************************************************************/
  public void onCreate(Bundle savedInstanceState){
    super.onCreate(savedInstanceState);

    // Creates the view from the XML file
    setContentView(R.layout.write_post_window);

    
    Spinner s = (Spinner) findViewById(R.id.color_flag_chooser);
    ArrayAdapter adapter = ArrayAdapter.createFromResource(
            this, R.array.color_flags_titles, android.R.layout.simple_spinner_item);
    adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
    s.setAdapter(adapter);
    
    
    // Set the View Click Listeners
    WSViewClickableListener viewClickableListener = new WSViewClickableListener(this);
    ((Button)findViewById(R.id.button_send_message)).setOnClickListener(viewClickableListener);



  }


  /***********************************************************************************************
  Creates the <code>Menu</code> from the XML file when the Options button is pressed
  @param menu the <code>Menu</code> <code>View</code> to populate with menu items
  @return <b>true</b> to display the menu, <br />
          <b>false</b> to keep the menu hidden.
  ***********************************************************************************************/
  public boolean onCreateOptionsMenu(Menu menu){
    MenuInflater inflater = getMenuInflater();
    inflater.inflate(R.menu.write_post_window,menu);
    return true;
  }


  /***********************************************************************************************
  Performs an action when a <code>MenuItem</code> is selected in the Options menu
  @param item the <code>MenuItem</code> that was selected
  @return <b>true</b> if the event was handled by this class<br />
          <b>false</b> if the event wasn't handled by this class, and thus should be passed on to
          the parent class for handling.
  ***********************************************************************************************/
  public boolean onOptionsItemSelected(MenuItem item){
    int id = item.getItemId();
    if (id == R.id.menu_write_post_window_about) {
      showAbout();
    }
    else if (id == R.id.menu_write_post_window_settings) {
      showSettings();
    }
    else if (id == R.id.menu_write_post_window_send_message) {
      runSendPostThread();
    }
    else {
    return false;
    }
    return true;
  }


  /***********************************************************************************************
  Hides the screen fields whenever the <code>Activity</code> is brought to the foreground
  ***********************************************************************************************/
  public void onResume(){
    super.onResume();
    
    
    // Set the selected Flag value
    int flagNumber = SharedHttpClient.getFlag();
    if (flagNumber <= 0){
      flagNumber = 1;
    }
    flagNumber--; // convert the flag number (1-200) to an array position (0-199)
    ((Spinner)findViewById(R.id.color_flag_chooser)).setSelection(flagNumber);
    

  }
  
  /***********************************************************************************************

  ***********************************************************************************************/
  public void runSendPostThread(){
    // "Loading" popup
    showDialog(DIALOG_LOADING);

    // call sendPost() in a Thread
    new Thread(new Runnable(){
      public void run(){
        sendPost();
        try {
      dismissDialog(DIALOG_LOADING);
        }
        catch (Throwable t){
        }
      }
      }).start();
  }
  
  /***********************************************************************************************

   ***********************************************************************************************/
  public Activity getThisActivity(){
    return this;
  }
  
  /***********************************************************************************************

  ***********************************************************************************************/
  public void sendPost(){
    
    try {
    
      // get the message input
      String postMessage = ((EditText)findViewById(R.id.tf_message)).getText().toString();
      if (postMessage == null || postMessage.length() <= 0){
        // no message
        this.runOnUiThread(new Runnable(){
          public void run(){
            WSAndroidPopup.showError(getThisActivity(),R.string.error_message_missing);
          }
          });
        return;
      }
      
      // get the flag input
      int flagNumber = ((Spinner)findViewById(R.id.color_flag_chooser)).getSelectedItemPosition();
      if (flagNumber == Spinner.INVALID_POSITION){
        flagNumber = 1;
      }
      else {
        flagNumber++; // the value from the spinner starts at 0, but the chumsgroup values start at 1
      }
      SharedHttpClient.setFlag(flagNumber);
    
    // 1. Send a POST to http://chumsgroup.com/messages, passing in authenticity_token, utf8="&#x2713;", message[content], message[user_id], message[flag_id], commit="Chum it!"
    String batchURL = "http://chumsgroup.com/messages";
    HttpPost postRequest = new HttpPost(batchURL);
    postRequest.setHeader("Accept","text/html"); // force html response
    postRequest.setHeader("Accept-Encoding","identity"); // force no-compression response (for 3G compat)

    // Create a new MultipartEntity for holding the POST data
    MultipartEntity entity = new MultipartEntity(HttpMultipartMode.BROWSER_COMPATIBLE);
    entity.addPart("authenticity_token",new StringBody(SharedHttpClient.getToken()));
    entity.addPart("utf8",new StringBody("&#x2713;"));
    entity.addPart("message[content]",new StringBody(postMessage));
    entity.addPart("message[user_id]",new StringBody(SharedHttpClient.getUserId()));
    entity.addPart("message[flag_id]",new StringBody(""+flagNumber));
    entity.addPart("commit",new StringBody("Chum it!"));

    postRequest.setEntity(entity);

    // Send the request
    DefaultHttpClient client = SharedHttpClient.getClient();
    HttpResponse response = client.execute(postRequest);

    // Get the response code
    if (response.getStatusLine().getStatusCode() != HttpStatus.SC_OK) {
      this.runOnUiThread(new Runnable(){
        public void run(){
          WSAndroidPopup.showError(getThisActivity(),R.string.error_posting_failure);
        }
        });
      return;
    }

    // 2. Hopefully get a webpage back as the response, indicating success.
    //String responseMessage = EntityUtils.toString(response.getEntity());
    //String temp = "";
    

    
      
    
    }
    catch (Throwable t){

    }
    
    // Close the activity, and return to the PostsWindow
    finish();
  }
  
  
  final int DIALOG_LOADING = 1;
  
  /***********************************************************************************************
  Overwritten to handle change of screen orientation while the dialog is showing
  ***********************************************************************************************/
  protected Dialog onCreateDialog(int id) {
    if(id == DIALOG_LOADING){
    ProgressDialog loadingDialog = new ProgressDialog(this);
    loadingDialog.setMessage("Sending Chum Message...");
    loadingDialog.setIndeterminate(true);
    loadingDialog.setCancelable(true);
    return loadingDialog;
    }

    return super.onCreateDialog(id);
    }
  
  /***********************************************************************************************
  Overwritten to close the dialog when clicking the 'back' button and returning to this activity
  ***********************************************************************************************/
  protected void onSaveInstanceState(Bundle outState) {
    try {
    dismissDialog(DIALOG_LOADING);
    } catch (Exception e) {
    //ignore error
    }
    super.onSaveInstanceState(outState);
    }
  
  
  /***********************************************************************************************
  Shows the about <code>Activity</code>
  ***********************************************************************************************/
  public void showAbout(){
    Intent aboutActivity = new Intent(getBaseContext(),WSAboutScreen.class);
    startActivity(aboutActivity);
  }


  /***********************************************************************************************
  Shows the settings <code>Activity</code>
  ***********************************************************************************************/
  public void showSettings(){
    Intent settingsActivity = new Intent(getBaseContext(),WSSettingsChanger.class);
    startActivity(settingsActivity);
  }

}